---
title: Material system
---

# Introduction to DuMuX- Material system

## Challenges in simulating porous media:

* Highly heterogeneous distribution of parameters and complex nonlinear material laws

* Strong interconnection of properties **--->** difficult to achieve modularity

## Modular structure:

<ins> User-defined parameters and relationships: </ins>

* Components
* Fluid system
* Solid system
* Binary coefficients
* Fluid-matrix interactions
* Chemistry

## <ins> Dumux-specific containers and solvers </ins>

- Fluid states
- Solid states
- Constraint solvers

# Material system: Component

## Component

<img src="img/component.png" width="200"/>

## <ins> What it does: </ins>

* **Thermodynamic relations** (e.g. molar mass, vapor pressure, density) of a **single chemical species** or a fixed mixture of species
* Provide a convenient way to access these quantities
* Not supposed to be used by models directly

## <ins> Example implementations: </ins>

* _H2O_ : pure water, properties by IAPWS-97 or simplified

* _Brine_ : water with a given salt concentration

# Material system: Fluid system

## Fluid system

<img src="img/fluidsystem.png" width="300"/>

## <ins> What it does: </ins>

Expresses the **thermodynamic relations between fluid quantities** (e.g. calculation of density or viscosity based on composition, fugacity coefficient based on temperature and pressure...)

## <ins> Example implementations: </ins>

* _TwoPImmiscible_ : two immiscible fluid phases

* _H2OAir_ : gas and liquid phase with components water and air

# Material system: Binary coefficients

## Binary coefficients

<img src="img/binarycoefficients.png" width="350"/>

## <ins> What it does: </ins>

  **Contains** data and equations required for binary mixtures (e.g. binary diffusion coefficients, coefficients needed for constitutive relationships (like Henry coefficient))

## <ins> Example implementations: </ins>

* _H2O_Air_ : Henry coefficient, gas diffusion coefficient, liquid diffusion coefficent for water and air

# Material system: Solid system

## Solid system

<img src="img/solidsystem.png" width="300"/>

## <ins> What it does: </ins>

Expresses the **thermodynamic properties of the solid matrix** (e.g. calculation of the solid density and solid heat capacity based on the composition...)

<ins> Note to solid system </ins>

_Specifying a solid system is only necessary if you work with a non-isothermal or mineralization model. If no solid system is specified in the problem file, the default is the inert solid phase with the constant component. For the constant component you can set properties in the input file._


## <ins> Implementations: </ins>

* _OneCSolid_ : inert solid matrix of one solid component (e.g. granite)
* _CompositionalSolidPhase_ : composed solid matrix of inert or reactive components (e.g. NaCl and granite)

# Material system: Fluid-matrix interactions

## Fluid-materix interactions

<img src="img/fluidmatrixinteractions.png" width="250"/>

## <ins> What it does: </ins>

* Description of the **interaction of the fluid phases with the porous medium** (e.g. capillary pressure-saturation and relative permeability relationships)
* Through modular adapters, regularization schemes can be imposed for extreme values

## <ins> Example implementations: </ins>

* Available parametrizations for the capillary pressure-saturation relationship are:
    * _Van Genuchten_
    * _Brooks Corey_

## Van-Genuchten
$\begin{equation}
p_c = \frac{1}{\alpha}\left(S_e^{-1/m} -1\right)^{1/n}
\end{equation}$

<img src="img/pc-Sw_VanGenuchten.png" width="500"/>

**--->** Necessary are here the empirical parameters $\alpha$ and $n$.

## Brooks-Corey
$\begin{equation}
p_c = p_d S_e^{-1/\lambda}
\end{equation}$

<img src="img/pc-Sw_BrooksCorey.png" width="500"/>

**--->** Necessary parameters are here the entry pressure $p_d$ and the shape factor $\lambda$.

# Material system: Chemistry

## Chemistry

<img src="img/chemistry.png" width="200"/>

## <ins> What it does: </ins>

Reactions between different components. There are extra models or they are realized with the introduction of a source term.

**Note:** _This contains some specific example implementations. One can implement specific things according to their need._

## <ins> Example implementations: </ins>
Expresses the **electrochemical models for a fuel cell application**:

* _Electrochemistry_ : for isothermal system
* _Electrochemistryni_ : for non-isothermal system

# Material system: Fluid state

## Fluid state

<img src="img/fluidstate.png" width="800"/>

## <ins> What it does: </ins>

 * **Stores** the complete thermodynamic configuration of a system at a given spatial and temporal position (e.g. saturation, mole fraction, enthalpy)
 * **Provides access** methods to all thermodynamic quantities (e.g. saturation, mole fraction, enthalpy)

## <ins> Example implementations: </ins>

 * _ImmiscibleFluidState_ : assumes immiscibility of the fluid phases. Phase compositions and fugacity coefficients do not need to be stored explicitly.
 * _CompositionalFluidState_ : assumes thermodynamic equilibrium, only a single temperature needs to be stored.

# Material system: Solid state

## Solid state

<img src="img/solidstate.png" width="800"/>

## <ins> What it does: </ins>

 * **Stores** the complete solid configuration of a system at a given spatial and temporal position (e.g. solid volume fractions, solid heat capacity)
 * **Provides** access methods to all solid quantities (e.g. porosity, density, temperature)

## <ins> Example implementations: </ins>

* _InertSolidState_ : assumes an inert solid phase. Solid volume fractions do not change. This is the **default**.

* _CompositionalSolidState_ : assumes a solid matrix composed out of two components. The volume fractions can change and properties such as heat capacity are adapted.

# Material system: Constraint Solver

## Constraint solver

<img src="img/constraintsolver.png" width="1000"/>

## <ins> What it does: </ins>

**Connects** the thermodynamic relations expressed by fluid systems with the thermodynamic quantities stored by fluid states (e.g. mole fraction, density)

## <ins> Example implementation: </ins>

_CompositionFromFugacities_ : takes all component fugacities, the temperature and pressure of a phase as input and calculates the phase composition

# Example: From component to fluid system

## Component **--->** fluid system

<img src="img/component-fluidsystem.png" width="500"/>

## Example: 2 phases, miscible

 * Components: _H2O_, _Air_
 * Fluid system: _TwoPTwoC_

## Include headers in properties file: components

```cpp
// The two-phase fluid system for compents water and air
#include <dumux/material/fluidsystems/h2oair.hh>
// The water component
#include <dumux/material/components/h2o.hh>
// The air component
#include <dumux/material/components/air.hh>
```

## Specify fluid system in properties file:

```cpp
template<class TypeTag>
struct FluidSystem<TypeTag, TTag::H2OAir>
{
private:
    using Scalar = GetPropType<TypeTag, Properties::Scalar>;
public:
    using type = FluidSystems::H2OAir<Scalar,
                 Components::TabulatedComponent
                 <Components::H2O<Scalar>>,
                 FluidSystems::H2OAirDefaultPolicy
                 </*fastButSimplifiedRelations=*/true>,
                 true /*useKelvinEquation*/>;
};
```


# Example: From component to solid system

## Example: 2 phases, miscible

 * Components: _CaO_, _CaO2H2_ (slaked lime)
 * Solid system: _OnePNCMin_

## Specify solid system in properties file:

```cpp
// The solid system
template<class TypeTag>
struct SolidSystem<TypeTag, TTag::ThermoChem>
{
    using Scalar = GetPropType<TypeTag, Properties::Scalar>;
    using ComponentOne = Components::ModifiedCaO<Scalar>;
    using ComponentTwo = Components::CaO2H2<Scalar>;
    using type = SolidSystems::CompositionalSolidPhase
                 <Scalar, ComponentOne, ComponentTwo>;
};
```


# Exercise

## <ins> Tasks: </ins>

1. Get familiar with the code
2. 2p model: Implement a new component (incompressible and compressible)
3. 2p2c model: Implement a new fluid system
4. Change wettability of the porous medium
5. Advanced: Use van Genuchten relationship with parameters: alpha = 0.0037 and alphalense = 0.00045, n = 4.7 and nlense = 7.3

**First step:** Go to <https://git.iws.uni-stuttgart.de/dumux-repositories/dumux-course/tree/master/exercises/exercise-fluidsystem> and check out the README

